-- Preemptive waits

-- Drop the session if it exists. 
IF EXISTS (
	SELECT * FROM sys.server_event_sessions
		WHERE [name] = N'PreemptiveWaits')
    DROP EVENT SESSION [PreemptiveWaits] ON SERVER
GO

-- Create the event session using dynamic SQL as
-- some wait_type numbers change from version to version
DECLARE @map		INT;
DECLARE @session	NVARCHAR (MAX) =

'CREATE EVENT SESSION [PreemptiveWaits] ON SERVER
ADD EVENT [sqlos].[wait_info_external]
(
    ACTION ([package0].[callstack])
    WHERE [opcode] = 1		-- Just the end wait events
	AND ([wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'MEMORY_ALLOCATION_EXT';

IF @map IS NOT NULL
	SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'MSQL_XP';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PERFORMANCE_COUNTERS_RWLOCK';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_FILESIZEGET';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_COM_CREATEACCESSOR';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_COM_GETDATA';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_COM_QUERYINTERFACE';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_COM_RELEASEACCESSOR';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_COM_RELEASEROWS';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_CREATEPARAM';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_DTC_ABORTREQUESTDONE';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_DTC_BEGINTRANSACTION';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_DTC_ENLIST';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OLEDB_SETPROPERTIES';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OLEDBOPS';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_ACCEPTSECURITYCONTEXT';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_ACQUIRECREDENTIALSHANDLE';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_AUTHENTICATIONOPS';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_AUTHORIZATIONOPS';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_AUTHZGETINFORMATIONFROMCONTEXT';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_AUTHZINITIALIZECONTEXTFROMSID';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_AUTHZINITIALIZERESOURCEMANAGER';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_CLOSEHANDLE';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_COMOPS';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_CREATEDIRECTORY';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_CREATEFILE';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_CRYPTACQUIRECONTEXT';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_CRYPTIMPORTKEY';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_CRYPTOPS';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_DECRYPTMESSAGE';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_DELETEFILE';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_DELETESECURITYCONTEXT';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_DEVICEIOCONTROL';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_DEVICEIOCONTROL';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_DTCOPS';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_ENCRYPTMESSAGE';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_FILEOPS';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_FINDFILE';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_FLUSHFILEBUFFERS';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_GENERICOPS';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_GETDISKFREESPACE';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_GETFILEATTRIBUTES';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_GETPROCADDRESS';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_GETVOLUMENAMEFORVOLUMEMOUNTPOINT';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_GETVOLUMEPATHNAME';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_INITIALIZESECURITYCONTEXT';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_LIBRARYOPS';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_LOADLIBRARY';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_LOOKUPACCOUNTSID';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_NETVALIDATEPASSWORDPOLICYFREE';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_PIPEOPS';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_QUERYCONTEXTATTRIBUTES';

IF @map IS NOT NULL
	SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_QUERYREGISTRY';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_REMOVEDIRECTORY';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_RSFXDEVICEOPS';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_SECURITYOPS';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_SETFILEVALIDDATA';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_SQMLAUNCH';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_REPORTEVENT';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_REVERTTOSELF';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_WAITFORSINGLEOBJECT';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_WRITEFILE';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_WRITEFILEGATHER';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_SB_STOPENDPOINT';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_SP_SERVER_DIAGNOSTICS';

IF @map IS NOT NULL
	SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_XE_CALLBACKEXECUTE';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_XE_GETTARGETSTATE';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_XE_SESSIONCOMMIT';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_XE_TARGETFINALIZE';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_XE_TARGETINIT';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_XE_TARGETINIT';

SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'RESERVED_MEMORY_ALLOCATION_EXT';

IF @map IS NOT NULL
	SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'XE_LIVE_TARGET_TVF';

IF @map IS NOT NULL
	SELECT @session = @session + CONVERT (NVARCHAR, @map) + ' AND [wait_type] != ';

SELECT @map = [map_key] FROM sys.dm_xe_map_values
WHERE [name] = N'wait_types' AND [map_value] = N'PREEMPTIVE_OS_DISCONNECTNAMEDPIPE';

SELECT @session = @session + CONVERT (NVARCHAR, @map);

SELECT @session = @session + 
'    )  
)
ADD TARGET [package0].[asynchronous_file_target]
    (SET FILENAME = N''C:\SQLskills\EE_PreemptiveWaits.xel'', 
    METADATAFILE = N''C:\SQLskills\EE_PreemptiveWaits.xem'')
WITH (max_dispatch_latency = 1 seconds)';

EXEC (@session);

-- Start the session
ALTER EVENT SESSION [PreemptiveWaits] ON SERVER
STATE = START;
GO

-- TF to allow call stack resolution
DBCC TRACEON (3656, -1);
GO

-- Run the workload

-- Get the callstacks from the file target
/*
SELECT
*
FROM
(SELECT
	[data].[value] (
		'(/event/data[@name=''wait_type'']/text)[1]', 'VARCHAR(MAX)') AS [Wait Type],
	[data],
	[data].[value] (
		'(/event/action[@name=''callstack'']/value)[1]',
			'VARCHAR(MAX)') AS [Call Stack]
FROM 
	(SELECT CONVERT (XML, [event_data]) AS [data]
	FROM sys.fn_xe_file_target_read_file
		(N'C:\SQLskills\EE_PreemptiveWaits*.xel',
		N'C:\SQLskills\EE_PreemptiveWaits*.xem', null, null)
	) [entries]
) [a]
ORDER BY [Wait Type]
GO
*/

-- Now stop the workload

/* Commented out so that the session can be set up by just hitting F5
-- Stop the event session
ALTER EVENT SESSION [PreemptiveWaits] ON SERVER
STATE = STOP;
GO

-- And clean up
IF EXISTS (
	SELECT * FROM sys.server_event_sessions
		WHERE [name] = N'PreemptiveWaits')
    DROP EVENT SESSION [PreemptiveWaits] ON SERVER
GO
*/
