-- Use this for latches only, not for waits

USE [master];
GO

IF DB_ID (N'SQLskillsLatchClassAnalysis') IS NOT NULL
BEGIN
       ALTER DATABASE [SQLskillsLatchClassAnalysis] SET SINGLE_USER WITH ROLLBACK IMMEDIATE;
	   DROP DATABASE [SQLskillsLatchClassAnalysis];
END
GO

CREATE DATABASE [SQLskillsLatchClassAnalysis];
GO

ALTER DATABASE [SQLskillsLatchClassAnalysis] SET RECOVERY SIMPLE WITH NO_WAIT;
GO

USE [SQLskillsLatchClassAnalysis];
GO

IF OBJECT_ID (N'tempdb..#SQLskillsResults') IS NOT NULL
       DROP TABLE #SQLskillsResults;

CREATE TABLE #SQLskillsResults (RowID INT IDENTITY, event_data XML);

INSERT INTO #SQLskillsResults (event_data)
SELECT CAST(event_data AS XML) AS event_data
FROM sys.fn_xe_file_target_read_file('C:\SQLskills\EE_Latches*xel', NULL, NULL, NULL)

SELECT
       RowID,
       event_data.value('(event/data[@name="class"]/text)[1]', 'varchar(128)') AS latch_class,
       event_data.value('(event/data[@name="duration"]/value)[1]', 'bigint') AS duration,
       event_data.value('(event/action[@name="callstack"]/value)[1]', 'varchar(8000)') AS callstack
INTO [SQLskillsLatchClassAnalysis].[dbo].[CallStackAnalysis]
FROM #SQLskillsResults
OPTION(RECOMPILE);

DROP TABLE #SQLskillsResults;
GO

-- Get just the latch_class, total_duration, signal_duration, and callstack and GROUP BY wait_type and callstack
SELECT
       [latch_class],
       SUM([duration]) AS [total_duration],
       COUNT(*) AS [occurrences],
       [callstack]
INTO [SQLskillsLatchClassAnalysis].[dbo].[CallStackAggregates]
FROM [SQLskillsLatchClassAnalysis].[dbo].[CallStackAnalysis]
GROUP BY [latch_class], [callstack];
GO

DROP TABLE [SQLskillsLatchClassAnalysis].[dbo].[CallStackAnalysis];
GO

CHECKPOINT;
GO

DBCC SHRINKDATABASE (N'SQLskillsLatchClassAnalysis');
GO

BACKUP DATABASE [SQLskillsLatchClassAnalysis]
TO  DISK = N'C:\SQLskills\SQLskillsLatchClassAnalysis.BAK'
WITH INIT;
GO

-- And put the backup file somewhere for me to download.
-- Please DO NOT email it to me.

-- If you want to see what was collected:
/*
SELECT latch_class, total_duration, occurrences,callstack
FROM SQLskillsLatchClassAnalysis.dbo.CallStackAggregates
ORDER BY latch_class
*/