-- Use this for waits only, not for latches

USE [master];
GO

IF DB_ID (N'SQLskillsWaitTypeAnalysis') IS NOT NULL
BEGIN
       ALTER DATABASE [SQLskillsWaitTypeAnalysis] SET SINGLE_USER WITH ROLLBACK IMMEDIATE;
	   DROP DATABASE [SQLskillsWaitTypeAnalysis];
END
GO

CREATE DATABASE [SQLskillsWaitTypeAnalysis];
GO

ALTER DATABASE [SQLskillsWaitTypeAnalysis] SET RECOVERY SIMPLE WITH NO_WAIT;
GO

USE [SQLskillsWaitTypeAnalysis];
GO

IF OBJECT_ID (N'tempdb..#SQLskillsResults') IS NOT NULL
       DROP TABLE #SQLskillsResults;

CREATE TABLE #SQLskillsResults (RowID INT IDENTITY, event_data XML);

INSERT INTO #SQLskillsResults (event_data)
SELECT CAST(event_data AS XML) AS event_data
-- uncomment out the appopriate line
--FROM sys.fn_xe_file_target_read_file('C:\SQLskills\EE_UncommonWaits*xel', NULL, NULL, NULL)
--FROM sys.fn_xe_file_target_read_file('C:\SQLskills\EE_PreemptiveWaits*xel', NULL, NULL, NULL)

SELECT
       RowID,
       event_data.value('(event/data[@name="wait_type"]/text)[1]', 'varchar(128)') AS wait_type,
       event_data.value('(event/data[@name="duration"]/value)[1]', 'bigint') AS duration,
       event_data.value('(event/data[@name="signal_duration"]/value)[1]', 'bigint') AS signal_duration,
       event_data.value('(event/action[@name="callstack"]/value)[1]', 'varchar(8000)') AS callstack
INTO [SQLskillsWaitTypeAnalysis].[dbo].[CallStackAnalysis]
FROM #SQLskillsResults
OPTION(RECOMPILE);

DROP TABLE #SQLskillsResults;
GO

-- Get just the wait_type, total_duration, signal_duration, and callstack and GROUP BY wait_type and callstack
SELECT
       [wait_type],
       SUM([duration]) AS [total_duration],
       SUM([signal_duration]) AS [total_signal_duration],
       COUNT(*) AS [occurrences],
       [callstack]
INTO [SQLskillsWaitTypeAnalysis].[dbo].[CallStackAggregates]
FROM [SQLskillsWaitTypeAnalysis].[dbo].[CallStackAnalysis]
GROUP BY [wait_type], [callstack];
GO

DROP TABLE [SQLskillsWaitTypeAnalysis].[dbo].[CallStackAnalysis];
GO

CHECKPOINT;
GO

DBCC SHRINKDATABASE (N'SQLskillsWaitTypeAnalysis');
GO

BACKUP DATABASE [SQLskillsWaitTypeAnalysis]
TO  DISK = N'C:\SQLskills\SQLskillsWaitTypeAnalysis.BAK'
WITH INIT;
GO

-- And put the backup file somewhere for me to download.
-- Please DO NOT email it to me.

-- If you want to see what was collected:
/*
SELECT wait_type, total_duration, total_signal_duration, occurrences,
	REPLACE(REPLACE(callstack, 'XeSosPkg::wait_info::Publish+138 [ @ 0+0x0
	SOS_Scheduler::UpdateWaitTimeStats+30c [ @ 0+0x0
	SOS_Task::PostWait+90 [ @ 0+0x0', ''), ' [ @ 0+0x0', '')
FROM SQLskillsWaitTypeAnalysis.dbo.CallStackAggregates
ORDER BY wait_type
*/